{

=pod

=head1 NAME

Blogger.pm

=head1 SUMMARY

    use Blogger;
    my $b = Blogger->new(appkey=>APPKEY);

    $b->BlogId(BLOGID);
    $b->Username(USERNAME);
    $b->Password(PASSWORD);

    $b->BlogId($b->GetBlogId(blogname=>'superfoobar'));

    my $user = $b->getUserInfo();
    print "Hello $user->{'firstname'}\n"; 

    # Get recent posts

    my ($ok,@p) = $b->getRecentPosts(numposts=>20);

    if (! $ok) {
	croak $b->LastError();
    }

    map { print "\t $_->{'postid'}\n"; } @p;

    # Post from a file

    my ($ok,@p) = $b->PostFromFile(file=>"/home/asc/tmp/blogger-test");

    if (! $ok) {
	croak $b->LastError();
    }

    # Deleting posts

    map {
	$b->deletePost(postid=>"$_") || croak $b->LastError();
    } @p;

    # Getting and setting templates

    my $t = $b->getTemplate(type => 'main');
    $b->setTemplate(type=>'main',template=>\$t) || croak $b->LastError();

    # New post

    my $txt = "hello world.";
    my $id = $b->newPost(postbody=>\$txt) || croak $b->LastError();

    # Get post

    my $post = $b->getPost($id) || croak $b->LastError();
    print "Text for last post was $post->{'content'}\n";

=head1 DESCRIPTION

TBW...

=cut

package Blogger;
use strict;

use vars qw ( $VERSION $AUTOLOAD );
$VERSION = 0.5.1.1;

use Error;
use FileHandle;
use Frontier::Client;

use constant BLOGGER_PROXY  => "http://plant.blogger.com/api/RPC2";
use constant MAX_POSTLENGTH => 65536;

my %entity = (
	      nbsp   => "&#160;",
	      iexcl  => "&#161;",
	      cent   => "&#162;",
	      pound  => "&#163;",
	      curren => "&#164;",
	      yen    => "&#165;",
	      brvbar => "&#166;",
	      sect   => "&#167;",
	      uml    => "&#168;",
	      copy   => "&#169;",
	      ordf   => "&#170;",
	      laquo  => "&#171;",
	      not    => "&#172;",
	      shy    => "&#173;",
	      reg    => "&#174;",
	      macr   => "&#175;",
	      deg    => "&#176;",
	      plusmn => "&#177;",
	      sup2   => "&#178;",
	      sup3   => "&#179;",
	      acute  => "&#180;",
	      micro  => "&#181;",
	      para   => "&#182;",
	      middot => "&#183;",
	      cedil  => "&#184;",
	      sup1   => "&#185;",
	      ordm   => "&#186;",
	      raquo  => "&#187;",
	      frac14 => "&#188;",
	      frac12 => "&#189;",
	      frac34 => "&#190;",
	      iquest => "&#191;",
	      Agrave => "&#192;",
	      Aacute => "&#193;",
	      Acirc  => "&#194;",
	      Atilde => "&#195;",
	      Auml   => "&#196;",
	      Aring  => "&#197;",
	      AElig  => "&#198;",
	      Ccedil => "&#199;",
	      Egrave => "&#200;",
	      Eacute => "&#201;",
	      Ecirc  => "&#202;",
	      Euml   => "&#203;",
	      Igrave => "&#204;",
	      Iacute => "&#205;",
	      Icirc  => "&#206;",
	      Iuml   => "&#207;",
	      ETH    => "&#208;",
	      Ntilde => "&#209;",
	      Ograve => "&#210;",
	      Oacute => "&#211;",
	      Ocirc  => "&#212;",
	      Otilde => "&#213;",
	      Ouml   => "&#214;",
	      times  => "&#215;",
	      Oslash => "&#216;",
	      Ugrave => "&#217;",
	      Uacute => "&#218;",
	      Ucirc  => "&#219;",
	      Uuml   => "&#220;",
	      Yacute => "&#221;",
	      THORN  => "&#222;",
	      szlig  => "&#223;",
	      agrave => "&#224;",
	      aacute => "&#225;",
	      acirc  => "&#226;",
	      atilde => "&#227;",
	      auml   => "&#228;",
	      aring  => "&#229;",
	      aelig  => "&#230;",
	      ccedil => "&#231;",
	      egrave => "&#232;",
	      eacute => "&#233;",
	      ecirc  => "&#234;",
	      euml   => "&#235;",
	      igrave => "&#236;",
	      iacute => "&#237;",
	      icirc  => "&#238;",
	      iuml   => "&#239;",
	      eth    => "&#240;",
	      ntilde => "&#241;",
	      ograve => "&#242;",
	      oacute => "&#243;",
	      ocirc  => "&#244;",
	      otilde => "&#245;",
	      ouml   => "&#246;",
	      divide => "&#247;",
	      oslash => "&#248;",
	      ugrave => "&#249;",
	      uacute => "&#250;",
	      ucirc  => "&#251;",
	      uuml   => "&#252;",
	      yacute => "&#253;",
	      thorn  => "&#254;",
	      yuml   => "&#255;",
	      );

my $entities = join('|', keys %entity);

=pod

=head1 CONSTRUCTOR METHODS

=head2 Blogger->new(%args)

Instantiate a new Blogger object.

Valid arguments are :

=over 4

=item B<appkey> 

String. The magic appkey for connecting to the Blogger XMLRPC server.

=item B<blogid>

String. The unique ID that Blogger uses for your weblog

=item B<username>

String. A valid username for blogid

=item B<password>

String. A valid password for the username/blogid pair.

=back

=cut

sub new {
    my $pkg = shift;
    
    my $self = {};
    bless $self,$pkg;

    $self->init(@_) || return undef;

    return $self;
}

sub init {
    my $self = shift;
    my $args = { @_ };

    $self->AppKey($args->{'appkey'});
    $self->Username($args->{'username'});
    $self->Password($args->{'password'});
    $self->BlogId($args->{'blogid'});

    $self->{'debug'}   = $args->{'debug'};
    $self->{"_posts"}  = [];

    return 1;
}

=pod 

=head1 BLOGGER API METHODS

=pod

=head2 $pkg->getUserInfo()

Returns a hash ref containing the following keys : userid, firstname, lastname, nickname, url, email.

=cut

sub getUserInfo {
    my $self = shift;

    my $user = {};

    eval {
	$user = $self->_Client->call(
				     "blogger.getUserInfo",
				     $self->_Client->string($self->AppKey()),
				     $self->_Client->string($self->Username()),
				     $self->_Client->string($self->Password()),				     
				     );
    };
    
    if ($@) {
      Error::Simple->record($@);
	return 0; 
    }

    return $user;
}

=pod

=head2 $pkg->getUserBlogs()

Fetch the I<blogid>, I<url> and I<blogName> for each of the Blogger blogs the current user is registered to.

Returns an array ref of hashes.

=cut

sub getUsersBlogs {
    my $self  = shift;
    my $blogs = [];

    eval { 
	$blogs = $self->_Client->call(
				  "blogger.getUsersBlogs",
				  $self->_Client->string($self->AppKey()),
				  $self->_Client->string($self->Username()),
				  $self->_Client->string($self->Password()),				     
				  );
    };

    if ($@) { 
      Error::Simple->record($@);
	return 0; 
    }
	
    return $blogs;
}

=head2 $pkg->newPost(%args)

Add a new post to the Blogger server. 

Valid arguments are :

=over 4

=item B<postbody> (required)

Scalar ref.

=item B<publish> 

Boolean.

=back

If the length of I<postbody> exceeds maximum length allowed by the Blogger servers -- 65,536 characters -- currently  the text will be chunked into smaller pieces are each piece will be posted separately.

Returns an array containing one, or more, post ids.

=cut

sub newPost {
    my $self = shift;
    my $args = { @_ };

    my $postbody = $args->{'postbody'};

    if (ref($postbody) ne "SCALAR") {
      Error::Simple->record("You must pass postbody as a scalar reference.");
	return 0;
    }

    if (($self->MaxPostLength()) && (length($$postbody) > $self->MaxPostLength())) {
	return $self->_PostInChunks(%$args);
    }

    my $publish = $args->{'publish'} ? 1 : 0;
    
    &_Encode($postbody);
    my $postid = undef;

    eval {
	$postid = $self->_Client->call(
				       "blogger.newPost",
				       $self->_Client->string($self->AppKey()),
				       $self->_Client->string($self->BlogId()),
				       $self->_Client->string($self->Username()),
				       $self->_Client->string($self->Password()),
				       $self->_Client->string($$postbody),
				       $self->_Client->boolean($publish),
				       );
    };

    if ($@) { 
      Error::Simple->record($@);
	return 0; 
    }
    
    return ($postid);
}

=pod

=head2 $pkg->getPost($postid)

Returns a hash ref, containing the following keys : userid, postid, content and dateCreated.

=cut

sub getPost {
    my $self   = shift;
    my $postid = shift;

    if (! $postid) {
      Error::Simple->record("You must specify a postid.");
	return 0;
    }
    
    my $post = undef;

    eval { 
	$post = $self->_Client->call(
				     "blogger.getPost",
				     $self->_Client->string($self->AppKey()),
				     $self->_Client->string($postid),
				     $self->_Client->string($self->Username()),
				     $self->_Client->string($self->Password()),
				     );
    };

    if ($@) {
      Error::Simple->record($@);
	return 0;
    }

    # See KNOWN ISSUES
    if ($post eq "0") {
      Error::Simple->record("Unable to locate post.");
	return 0;
    }

    $post->{'dateCreated'} = $post->{'dateCreated'}->repr();
    return $post;
}


=pod

=head2 $pkg->getRecentPosts(%args)

Fetch the latest (n) number of posts for a given blog. The most recent posts are returned first.

Valid arguments are 

=over 4

=item B<numposts>

Int. If no argument is passed to the method, default is 1.

"NumberOfPosts is limited to 20 at this time. Let me know if this 
gets annoying. Letting this number get too high could result in some 
expensive db access, so I want to be careful with it." --Ev

=back

Returns true or false, followed by an array of hash refs. Each hash ref contains the following keys : postid,content,userid,dateCreated

=cut

sub getRecentPosts {
    my $self = shift;
    my %args = @_;

    my $num   = (defined $args{'numposts'}) ? $args{'numposts'} : 1;
    my $posts = [];
    
    unless ($num =~ /^(\d+)$/) {
      Error::Simple->record("Argument $args{'numposts'} isn't numeric.");
	return 0;
    }

    unless (($num >= 1) && ($num <= 20)) {
      Error::Simple->record("You must specify 'numposts' as an integer between 1 and 20.");
	return 0;
    }

    eval { 
	$posts = $self->_Client->call(
				      "blogger.getRecentPosts",
				      $self->_Client->string($self->AppKey()),
				      $self->_Client->string($self->BlogId()),
				      $self->_Client->string($self->Username()),
				      $self->_Client->string($self->Password()),
				      $self->_Client->int($num),
				      );
    };
    
    if ($@) { 
      Error::Simple->record($@);
	return 0; 
    }
    
    # Frontier::Client returns the date as an object
    # so we call the objects "represent" method on
    # itself to get the actual value of the date.

    map { $_->{'dateCreated'} = $_->{'dateCreated'}->repr(); } @$posts;

    return (1,@$posts);
}

=pod

=head2 $pkg->editPost(%args)

Update the Blogger database. Set the body of entry $postid to $body.

Valid arguments are :

=over 4

=item B<postbody> (required)

Scalar ref or a valid filehandle.

=item B<postid> (required)

String.

=item B<publish> 

Boolean.

=back

If the length of I<postbody> exceeds maximum length allowed by the Blogger servers -- 65,536 characters -- currently  the text will be chunked into smaller pieces are each piece will be posted separately.

Returns an array containing one, or more, post ids.

=cut

sub editPost {
    my $self = shift;
    my $args = { @_ };

    my $postbody = $args->{'postbody'};
    my $postid   = $args->{'postid'};

    if (! $postid) { 
      Error::Simple->record("You must specify a postid.");
	return 0; 
    }
    
    if (ref($postbody) ne "SCALAR") {
      Error::Simple->record("You must pass postbody as a scalar reference.");
	return 0;
    }
    
    if (($self->MaxPostLength()) && (length($$postbody) > $self->MaxPostLength())) {
	return $self->_PostInChunks(%$args);
    }

    my $publish = ($args->{'publish'}) ? 1 : 0;

    &_Encode($postbody);

    my $pid = undef;

    eval {
	$pid = $self->_Client->call(
				    "blogger.editPost",
				    $self->_Client->string($self->AppKey()),
				    $self->_Client->string($postid),
				    $self->_Client->string($self->Username()),
				    $self->_Client->string($self->Password()),
				    $self->_Client->string($$postbody),
				    $self->_Client->boolean($publish),
				    );
    };

    if ($@) { 
      Error::Simple->record($@);
	return 0; 
    }
    
    return ($pid);
}

=pod

=head2 $pkg->deletePost(%args) 

Delete a post from the Blogger server.

Valid arguments are 

=over

=item B<postid> (required)

String.

=item B<publish> 

Boolean.

=back

Returns true or false.

=cut

sub deletePost {
    my $self = shift;
    my $args = { @_ };

    my $postid = $args->{'postid'};

    if (! $postid) {
      Error::Simple->record("No post id.");
	return 0;
    }

    my $publish = ($args->{'publish'}) ? 1 : 0;
    my $r       = undef;

    eval {
	$r = $self->_Client->call(
				  "blogger.deletePost",
				  $self->_Client->string($self->AppKey()),
				  $self->_Client->string($postid),
				  $self->_Client->string($self->Username()),
				  $self->_Client->string($self->Password()),
				  $self->_Client->boolean($publish),
				  );
    };

    if ($@) { 
      Error::Simple->record($@);
	return 0; 
    }

    return $r;
}

=pod 

=head2 $pkg->setTemplate(%args)

Set the body of the template matching type I<$type>.

<quote src = "ev">template is the HTML (XML, whatever -- Blogger can output any sort of text). Must contain opening and closing <Blogger> tags to be valid and accepted.</quote>

Valid arguments are 

=over 4

=item B<template>  (required)

Scalar ref.

=item B<type> (required)

String. Valid types are "main" and "archiveIndex"

=back

Returns true or false.

=cut

sub setTemplate {
    my $self = shift;
    my $args = { @_ };
    
    my $template = $args->{'template'};
    my $type     = $args->{'type'};

    if (ref($template) ne "SCALAR") {
      Error::Simple->record("You must pass template as a scalar reference.");
	return 0;
    }
    
    unless ($type =~ /^(main|archiveIndex)$/) {
      Error::Simple->record("Valid template types are 'main' and 'archiveIndex'.");
	return 0;
    }

    # see also : The Perl Cookbook, chapter 6.15
    unless ($$template =~ /(<Blogger>)[^<]*(?:(?! <\/?Blogger>)<[^<]*)*(<\/Blogger>)/m) {
      Error::Simple->record("Your template must contain opening and closing <Blogger> tags.");
	return 0;
    }

    &_Encode($template);
    
    my $r = undef;

    eval {
	$r = $self->_Client->call(
				  "blogger.setTemplate",
				  $self->_Client->string($self->AppKey()),
				  $self->_Client->string($self->BlogId()),
				  $self->_Client->string($self->Username()),
				  $self->_Client->string($self->Password()),
				  $self->_Client->string($$template),
				  $self->_Client->string($type),
				  );
    };

    if ($@) { 
      Error::Simple->record($@);
	return 0; 
    }

    return $r;
}

=pod

=head2 $pkg->getTemplate(%args)

Fetch the body of the template matching type I<$type>.

Valid types are 

=over 4

=item B<type> (required)

String. Valid types are "main" and "archiveIndex"

=back

Returns a string.

=cut

sub getTemplate {
    my $self = shift;
    my $args = { @_ };
    
    unless ($args->{'type'} =~ /^(main|archiveIndex)$/) {
      Error::Simple->record("Valid template types are 'main' and 'archiveIndex'.");
	return 0;
    }
    
    my $template = undef;

    eval { 
	$template = $self->_Client->call(
					 "blogger.getTemplate",
					 $self->_Client->string($self->AppKey()),
					 $self->_Client->string($self->BlogId()),
					 $self->_Client->string($self->Username()),
					 $self->_Client->string($self->Password()),
					 $self->_Client->string($args->{'type'}),
					 );
    };
    
    if ($@) { 
      Error::Simple->record($@);
	return 0; 
    }
    
    return $template;
}

=pod

=head1 BLOGGER DOT PM METHODS

=cut

=pod

=head2 $pkg->Proxy()

Return the URI of the Blogger XML-RPC proxy

=cut

sub Proxy {
    return BLOGGER_PROXY;
}

=pod

=head2 $pkg->MaxPostLength()

Return the maximum number of characters a single post may contain.

=cut

sub MaxPostLength {
    return MAX_POSTLENGTH;
}

=head2 $pkg->AppKey($key)

Get/set the magic appkey

=head2 $pkg->BlogId($id)

Get/set the blogid

=head2 $pkg->Username($username)

Get/set the username

=head2 $pkg->Password($password)

Get/set the password

=head2 $pkg->GetBlogId(%args) 

Return the unique blogid for I<$args{'blogname'}>.

Valid arguments are 

=over 4

=item B<blogname> => string.

=back.

Returns a string. If no blogname is specified, the current blogid for the object is returned.

=cut

sub GetBlogId {
    my $self = shift;
    my $args = { @_ };

    my $blogid = undef;

    if (! $args->{'blogname'}) {
	return $self->{'_blogid'};
    }

    my $blogs = $self->getUsersBlogs();

    foreach my $b (@$blogs) {
	if ($b->{'blogName'} eq $args->{'blogname'}) {
	    $blogid = $b->{'blogid'};
	    last;
	}
    }
    
    return $blogid;
}

=pod

=head2 $pkg->PostFromFile(%args)

Open a filehandle, and while true, post to Blogger. If the length of the amount read from the file exceeds the per-post limit assigned by the Blogger servers -- currently 65,536 characters -- the contents of the file will be posted in multiple "chunks".

Valid arguments are 

=over 4 

=item B<file> (required)

/path/to/file

=item B<postid> 

String.

=item B<publish> 

Boolean.

=item B<tail> 

Boolean.

If true, the method will not attempt to post data whose length exceeds the limit set by the Blogger server in the order that the data is read. Translation : last in becomes last post becomes the first thing you see on your weblog.

=back

If a I<postid> argument is present, the method will call the Blogger API I<editPost> method with postid. Otherwise the method will call the Blogger API I<newPost> method.

Returns true or false, followed by an array of zero, or more, postids. 

=cut

sub PostFromFile {
    my $self = shift;
    my $args = { @_ };

    if (! -f $args->{'file'}) {
      Error::Simple->record("Not a file.");
	return 0;
    }

    my $fh = FileHandle->new();
    my $ok = $fh->open("<$args->{'file'}");

    if (! $ok) {
      Error::Simple->record("Failed to open file : $!");
	return 0;
    }

    my $method  = ($args->{'postid'}) ? "editPost" : "newPost";

    if (! $args->{'tail'}) {

	local $/;
	undef $/;

	my $postbody = <$fh>;
	$fh->close();

	return $self->$method(postbody=>\$postbody,%$args);
    }

    my $post    = "";
    my @postids = ();

    while (<$fh>) {
	my $line = $_;
	chomp $line;

	$post  .= $line;
	my $len = length($post);

	if ($len > MAX_POSTLENGTH) {

	    my $postbody  = substr($post,0,MAX_POSTLENGTH);
	    my $remainder = &_TrimPostBody(\$postbody);

	    my ($pid) = $self->$method(
				       postbody => \$postbody,
				       postid   => $args->{'postid'},
				       publish  => $args->{'publish'},
				       );

	    if (! $pid) {
		$fh->close();
		
	      Error::Simple->record("Encountered an error posting. Exiting prematurely.");
		return (0,@postids);
	    }
	    
	    push(@postids,$pid);
	    $post = $remainder.substr($post,MAX_POSTLENGTH,$len);
	}
    }

    $fh->close();

    if (! $post) {
      Error::Simple->record("Failed to read any data from file.");
	return 0;
    }

    my ($pid) = $self->$method(
			       postbody => \$post,
			       postid   => $args->{'postid'},
			       publish  => $args->{'publish'},
			       );
    
    if (! $pid) {
      Error::Simple->record("Encountered an error posting last chunk : ".Error->prior());
	return (0, @postids);
    } 

    push (@postids,$pid);    
    return (1,@postids);
}

=pod

=head2 $pkg->LastError()

Fetch the last recorded error message.

Returns a string.

=cut

sub LastError {
    my $e = Error->prior();
    chomp $e;
    return $e;
}

=pod

=head1 PRIVATE METHODS

=cut

sub _PostInChunks {
    my $self = shift;
    my $args = { @_ };

    my $caller  = (caller(1))[3];
    my @chunks  = ();
    my @postids = ();

    unless ($caller =~ /^(Blogger::)(editPost|newPost)$/) {
      Error::Simple->record("$caller is not a valid caller for this method.");
	return 0;
    }

    my $text = $args->{"postbody"};
    
    while ( $$text ) {
	my $chunk     = substr($$text,0,MAX_POSTLENGTH);
	my $remainder = &_TrimPostBody(\$chunk);

	$$text = $remainder.substr($$text,length($chunk),length($$text));

	# Since Blogger posts are chronological, we add chunks 
	# to the top of the stack. That way, the 'end' pieces get
	# added first and, in the end, the text will be displayed
	# in the order it was written.
	# 20010813 (asc)

	unshift (@chunks, \$chunk);
    }
    
    map { 
	$args->{"postbody"} = $_;
	push(@postids, $self->$caller(%$args)); 
    } @chunks;

    return @postids;
}

sub _TrimPostBody {
    my $body = shift;

    if (ref($body) ne "SCALAR") {
      Error::Simple->report("Input must be a scalar ref.");
	return undef;
    }
    
}

sub _Encode {
    my $text = shift;
    $$text =~ s/&(?!(#[0-9]+|#x[0-9a-fA-F]+|\w+);)/&amp;/gm;
    $$text =~ s/&($entities);/$entity{$1}/gm;
    $$text =~ s/</&lt;/gm;		     
}

sub _Client {
    my $self = shift;

    $self->{"_xmlrpc"} ||= 
      Frontier::Client->new( url => $self->Proxy() , debug => $self->{'debug'})
	  || Error->throw(-text=>$!);

    return $self->{"_xmlrpc"};
}

sub DESTROY {
    return 1;
}

sub AUTOLOAD {
    my $self = shift;

    $AUTOLOAD =~ s/.*:://;

    if ($AUTOLOAD eq "Publish") {
      Error::Simple->record("This method has been deprecated.");
	return undef;
    }

    unless ($AUTOLOAD =~ /^(Proxy|AppKey|BlogId|Username|Password)$/) {
      Error::Simple->record("Unkown method '$AUTOLOAD' called. Skipping.");
	return undef;
    }

    my $property = lc "_".$AUTOLOAD;

    if (my $arg = shift) { $self->{ $property } = $arg; }
   
    if ($AUTOLOAD eq "Proxy") {
	$self->{"_xmlrpc"} = undef;
    }

    return $self->{ $property };
}

return 1;

=head1 KNOWN ISSUES

=over 4

=item B<Delays>

It remains uncertain how long a program needs to wait between the time that a new post is submitted to the Blogger servers and that that may post may be acted upon. The applies to the Blogger API I<getPost>, I<editPost> and I<deletePost> methods equally. Anything under a 10 second will often result in a "post not found" fault. A delay of 10 seconds or more is usually successful. Your mileage may vary.

=item B<setTemplate()>

<quote src = "ev">There are some blogs for which setTemplate will return a permission denied error. Newly created blogs will work. Sufficiently older blogs will work. A meanwhile work-around: edit the template through Blogger UI first.</quote>

=back

=cut

=head1 TO DO

=over 4

=item B<Untainting> 

Some degree of basic untainting so we're not constantly polluting the network with XMLRPC calls that are going to fail. 

=item B<Memoization>

What can be memoized safely?

=item B<Long-posts>

Add some kind of logic to the Blogger.pm I<_PostInChunks> and I<PostFromFile> methods for chunking in an intelligent fashion, rather than simply chopping words in mid flow.

Add the ability for a user to specify a max post length, so long as it is less than the default.

=back

=head1 VERSION

0.5.1.1

=head1 DATE

September 03, 2001

=head1 AUTHOR

Aaron Straup Cope

=head1 CHANGES

=head2 0.5.1.1

=over 4

=item

Made sure all the publish flags are passed as "0" or "1" and not "true/false". There was some confusion on my part on what, exactly, the spec defines and what the various server implementations can deal with.

=back

=head2 0.5.1

=over 4

=item

Added Blogger API I<getUserInfo> method.

=item 

Updated POD.

=back

=head2 0.5

=over 4

=item 

Added Blogger API I<getPost> method.

=item

Updated POD.

=back

=head2 0.4.6.1

=over 4

=item

Added conditional, where necessary, to see if a maximum post length is applicable.

=back

=head2 0.4.6

=over 4

=item

Added Blogger.pm I<BLOGGER_PROXY> constant.

=item

Added Blogger.pm I<Proxy> and I<MaxPostPostLength> accessors for corresponding constants. Previously, these values were either read from a scalar constant or an AUTOLOAD method. The change allows [insert blogger-mimicking interface here] subclasses to override the methods and specify an approriate value.

=item 

Updated POD

=back

=head2 0.4.5

=item 

Added Blogger API I<getRecentPosts> method.

=item

Updated POD

=head2 0.4.4.1

=over 4

=item

Clarified a few error messages;

=item

Fixed remaining instances of "Error::Simple->record() and return 0" in &AUTOLOAD

=back

=head2 0.4.4

=item 

Added use of Error.pm

=item 

Added Blogger.pm I<LastError> method.

=item 

Wrapped Frontier::Client method calls in eval statements to prevent unnecessary die-ing.

=item 

Added stub function and methods calls for Blogger.pm private I<_TrimPostBody> method.

=item 

Changed return value of Blogger.pm I<PostFromFile> to (boolean, array)

=item 

Updated POD.

=back

=head2 0.4.3

=over 4

=item 

Made sure all Blogger.pm methods begin with title case.

=back

=head2 0.4.2

=item 

Added private Blogger.pm I<_Encode> method. Code courtesy of Matt Sergeant's rssmirror.pl script. I<Someone, give this guy a YAS grant.>

=item

Added --tail flag to Blogger.pm I<PostFromFile> method.

=item 

Fixed a bug in Blogger.pm I<_PostInChunks> method where I would end up subscripting outside of the string.

=back

=head2 0.4.1

=over 4

=item

Added idiot-level escaping of entities in Blogger.pm I<newPost> and I<editPost> methods. Duh.

=back

=head2 0.4

=over 4

=item

Switched to named-based pair arguments.

=item

Added Blogger API I<deletePost> method.

=item 

Added the Blogger.pm I<PostFromFile> method. I<Experimental>

=item

Changed Blogger.pm MAX_POSTLENGTH constant.

=item

Updated POD

=back

=head2 0.3.1

=over 4

=item

Updated POD

=back

=head2 0.3

=over 4

=item

Added the Blogger API I<getTemplate> and I<setTemplate> methods.

=item 

Added the Blogger.pm I<_PostInChunks> method.

=item 

Changed the order in which parameters are passed to I<editPost>.

=item 

Changed the return value of both the Blogger API I<newPost> and I<editPost> methods.

=back

=head2 0.2

=over 4

=item 

Added the Blogger API I<getUsersBlogs> and I<editPost> methods.

=item 

Adde the Blogger.pm I<GetBlogId> method.

=item 

Removed the Blogger.pm I<Publish> method.

=item

Modifed the Blogger API I<newPost> method to accept the option to publish. 

=back

=head2 0.1

=over 4

=item

Initial setup. 

=item

Added the constructor methods.

=item

Added the Blogger API I<newPost> method.

=item

Added the Blogger.pm I<Publish> method.

=back

=head1 SEE ALSO

L<Frontier::Client>

http://plant.blogger.com/api/

http://xml.sergeant.org/download/rssmirror.pl

=head1 LICENSE

Copyright 2001, Aaron Straup Cope.

This is free software, you may use it and distribute it under the
same terms as Perl itself.

=cut

}
